<?php
/**
 * 2008-2016 Chronopostfood SAS
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to chronopostfood@adexos.fr so we can send you a copy immediately.
 *
 * @author    Adexos for Chronopostfood <chronopostfood@adexos.fr>
 * @copyright 2008-2016 Chronopostfood SAS
 * @license   http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 */

if (!defined('_CF_MYDIR_')) define('_CF_MYDIR_', dirname(__FILE__));
require('../../config/config.inc.php');

if (!Tools::getIsset('orderid') && !Tools::getIsset('orders') && !Tools::getIsset('orderid')) die('<h1>Informations de commande non transmises</h1>');

require_once('chronopostfood.php');
include('libraries/ShippingServiceWSService.php');
include_once _CF_MYDIR_.'/libraries/PointRelaisServiceWSService.php';


$multi = array();

if (!Tools::getIsset('shared_secret') || Tools::getValue('shared_secret') != Configuration::get('CHRONOFOOD_SECRET'))
	die('Secret does not match.');


if (Tools::strlen(Configuration::get('CHRONOFOOD_GENERAL_ACCOUNT')) < 8)
	die('Erreur : veuillez configurer le module avant de procéder à l\'édition des étiquettes.');

$return = false;

if (Tools::getIsset('multi'))
{
	$multi = Tools::getValue('multi');
	$multi = Tools::jsonDecode($multi, true);
} 
else $multi = array();

if (Tools::getIsset('orders'))
{
	$orders = Tools::getValue('orders');
	$orders = explode(';', $orders);
} 
else 
{
	$orders = array(Tools::getValue('orderid'));
	if (Tools::getIsset('return')) $return = true;
	if (Tools::getIsset('multiOne')) $multi = array($orders[0]=>Tools::getValue('multiOne'));
}

if (Tools::getIsset('expiration_date') && Tools::getValue('expiration_date') != '')
{
    $expirationDate = Tools::jsonDecode(Tools::getValue('expiration_date'), true);

    if (!is_array($expirationDate)) {
        $expirationDate = array(
            $orders[0] => Tools::getValue('expiration_date')
        );
    }
} else {
    die('Vous devez choisir une date limite de consommation');
}


if (count($orders) == 0) die('<h1>Aucune commande sélectionnée</h1>');
require_once('libraries/PDFMerger.php');
@$pdf = new PDFMerger;

foreach ($orders as $orderid)
{
	if (array_key_exists($orderid, $multi))
		$nb = $multi[$orderid];
	else $nb = 1;

    if (array_key_exists($orderid, $expirationDate) && $expirationDate[$orderid] != '') {
        $orderExpirationDate = $expirationDate[$orderid];
        $orderExpirationDate = new DateTime($orderExpirationDate);
        $dayAfter = new DateTime('+1 day');
        if ($orderExpirationDate->format('Y-m-d') < $dayAfter->format('Y-m-d')) {
            // Expiration date MUST be present for fresh products, skip
            Chronolog::log("Expiration date invalid for order $orderid", Chronolog::ERROR);
            continue;
        }
    }
    else {
        // Expiration date MUST be present for fresh products, skip
        Chronolog::log("Expiration date missing for order $orderid", Chronolog::ERROR);
        continue;
    }

	$totalnb = $nb;

	while ($nb > 0)
	{
		$file = tempnam('temp', 'CHR');
		$fp = fopen($file, 'w');

		$lt = createLT($orderid, $orderExpirationDate, $totalnb, $return);
		if ($lt === null) 
		{ 
			/* error, skip it */
			$nb--;
			continue;
		}
		fwrite($fp, $lt);
		fclose($fp);
		chmod($file, 0644);

		if (file_exists('custom_pdf/config.json'))
		{
			/* personnalize */
			include_once('custom_pdf/CustomPDF.php');
			$cust = new CustomPDF($file, $orderid);
			$cust->generate();
		}

		$pdf->addPDF($file, 'all');
		$nb--;
	}
}

try {
	$pdf->merge('download', 'Chronopostfood-LT-'.date('Ymd-Hi').'.pdf');
} catch(Exception $e)
{
	echo '<p>Le fichier généré est invalide.</p>';
	echo '<p>Vérifiez la configuration du module et que les commandes visées disposent d\'adresses de livraison valides.</p>';
}


function createLT($orderid, $orderExpirationDate = null, $totalnb = 1, $isReturn = false)
{
    if ($orderExpirationDate == null) {
        echo '<script>alert(\'Vous devez renseigner la DLC\');history.back();</script>';
        exit();
    }

	$o = new Order($orderid);
	$a = new Address($o->id_address_delivery);
	$cust = new Customer($o->id_customer);

	// at least 2 skybills for orders >= 30kg
	$o = new Order($orderid);
	if ($o->getTotalWeight() * Configuration::get('CHRONOFOOD_GENERAL_WEIGHTCOEF') >= 30 && $totalnb == 1)
	{
		echo '<script>alert(\'Vous devez générer au moins 2 étiquettes pour les commandes de plus de 30kg\');history.back();</script>';
		exit();
	}

	$recipient = new recipientValue();
	$recipient->recipientAdress1 = Tools::substr($a->address1, 0, 35);
	$recipient->recipientAdress2 = Tools::substr($a->address2, 0, 35);
	$recipient->recipientCity = Tools::substr($a->city, 0, 30);
	$recipient->recipientCivility = 'M';
	$recipient->recipientContactName = Tools::substr($a->firstname.' '.$a->lastname, 0, 35);
	$c = new Country($a->id_country);
	$recipient->recipientCountry = $c->iso_code;
	$recipient->recipientName = Tools::substr($a->company, 0, 35);
	$recipient->recipientName2 = Tools::substr($a->firstname.' '.$a->lastname, 0, 35);
	$recipient->recipientZipCode = $a->postcode;
	$recipient->recipientPhone = $a->phone_mobile == null ? $a->phone : $a->phone_mobile;
	$recipient->recipientMobilePhone = $a->phone_mobile;
	$recipient->recipientEmail = $cust->email;

	if ($isReturn) 
	{
		$recipient->recipientAdress1 = Configuration::get('CHRONOFOOD_SHIPPER_ADDRESS');
		$recipient->recipientAdress2 = Configuration::get('CHRONOFOOD_SHIPPER_ADDRESS2');
		$recipient->recipientCity = Configuration::get('CHRONOFOOD_SHIPPER_CITY');
		$recipient->recipientCivility = Configuration::get('CHRONOFOOD_SHIPPER_CIVILITY');
		$recipient->recipientContactName = Configuration::get('CHRONOFOOD_SHIPPER_CONTACTNAME');
		$recipient->recipientCountry = 'FR';
		$recipient->recipientName = Configuration::get('CHRONOFOOD_SHIPPER_NAME');
		$recipient->recipientName2 = Configuration::get('CHRONOFOOD_SHIPPER_NAME2');
		$recipient->recipientZipCode = Configuration::get('CHRONOFOOD_SHIPPER_ZIPCODE');
	}


	$esd = new esdValue();
	$esd->specificInstructions = 'aucune';
	//$esd->retrievalDateTime=date('Y-m-d\TH:i:s', $retrievalDateTime);
	//$esd->closingDateTime=>date('Y-m-d\TH:i:s', $closingDateTime);
	/*$esd->height=(float)$_POST['height'];
	$esd->width=(float)$_POST['width'];
	$esd->length=(float)$_POST['length'];*/

	$esd->height = '';
	$esd->width = '';
	$esd->length = '';

	$header = new headerValue();
	$params = new shipping();
	$skybill = new skybillValue();
	$skybill->evtCode = 'DC';
	$skybill->objectType = 'MAR';
	$skybill->productCode = '2R'; // CHRONOFOOD 13

	if (Tools::getIsset('advalorem') && Tools::getValue('advalorem') == 'yes')
		$skybill->insuredValue = (int)Tools::getValue('advalorem_value');

	$header->accountNumber = Configuration::get('CHRONOFOOD_GENERAL_ACCOUNT');
	$header->subAccount = Configuration::get('CHRONOFOOD_GENERAL_SUBACCOUNT');
	$params->password = Configuration::get('CHRONOFOOD_GENERAL_PASSWORD');

	$header->idEmit = 'PREST';

	$shipper = new shipperValue(); 
	$shipper->shipperAdress1 = Configuration::get('CHRONOFOOD_SHIPPER_ADDRESS');
	$shipper->shipperAdress2 = Configuration::get('CHRONOFOOD_SHIPPER_ADDRESS2');
	$shipper->shipperCity = Configuration::get('CHRONOFOOD_SHIPPER_CITY');
	$shipper->shipperCivility = Configuration::get('CHRONOFOOD_SHIPPER_CIVILITY');
	$shipper->shipperContactName = Configuration::get('CHRONOFOOD_SHIPPER_CONTACTNAME');
	$shipper->shipperCountry = 'FR';
	$shipper->shipperName = Configuration::get('CHRONOFOOD_SHIPPER_NAME');
	$shipper->shipperName2 = Configuration::get('CHRONOFOOD_SHIPPER_NAME2');
	$shipper->shipperZipCode = Configuration::get('CHRONOFOOD_SHIPPER_ZIPCODE');
	$shipper->shipperPhone = Configuration::get('CHRONOFOOD_SHIPPER_PHONE');
	$shipper->shipperMobilePhone = Configuration::get('CHRONOFOOD_SHIPPER_MOBILE');


	if ($isReturn)
	{
		$shipper = new shipperValue(); 
		$shipper->shipperAdress1 = Tools::substr($a->address1, 0, 35);
		$shipper->shipperAdress2 = Tools::substr($a->address2, 0, 35);
		$shipper->shipperCity = Tools::substr($a->city, 0, 30);
		$shipper->shipperCivility = 'M';
		$shipper->shipperContactName = Tools::substr($a->firstname.' '.$a->lastname, 0, 35);
		$shipper->shipperCountry = 'FR';
		$shipper->shipperName = Tools::substr($a->company, 0, 35);
		$shipper->shipperName2 = Tools::substr($a->firstname.' '.$a->lastname, 0, 35);
		$shipper->shipperZipCode = $a->postcode;
		$shipper->shipperPhone = $a->phone;
		$shipper->shipperMobilePhone = $a->phone_mobile;
	}

	$customer = new customerValue();
	$customer->customerAdress1 = Configuration::get('CHRONOFOOD_CUSTOMER_ADDRESS');
	$customer->customerAdress2 = Configuration::get('CHRONOFOOD_CUSTOMER_ADDRESS2');
	$customer->customerCity = Configuration::get('CHRONOFOOD_CUSTOMER_CITY');
	$customer->customerCivility = Configuration::get('CHRONOFOOD_CUSTOMER_CIVILITY');
	$customer->customerContactName = Configuration::get('CHRONOFOOD_CUSTOMER_CONTACTNAME');
	$customer->customerCountry = 'FR';
	$customer->customerName = Configuration::get('CHRONOFOOD_CUSTOMER_NAME');
	$customer->customerName2 = Configuration::get('CHRONOFOOD_CUSTOMER_NAME2');
	$customer->customerZipCode = Configuration::get('CHRONOFOOD_CUSTOMER_ZIPCODE');

	$ref = new refValue();
	$ref->recipientRef = $a->postcode;

	// CARRIER-SPECIFIC
	switch ($o->id_carrier)
	{
		case Configuration::get('CHRONOFOOD_CARRIER_ID'):
			$skybill->productCode = Chronopostfood::$productCodes['CHRONOFOOD_CARRIER_ID'];
			if (Configuration::get('CHRONOFOOD_BAL_ENABLED') == 1 && !$isReturn)
				$skybill->productCode = '58'; // CHRONO 13 + BAL
		break;
	}

	$ref->shipperRef = sprintf('%06d', $orderid);

	$skybill->service = '0';

	if ($o->id_carrier == Configuration::get('CHRONOFOOD_CARRIER_ID'))
		{

		// Called from hookAdminOrder
		if (Tools::getIsset('shipSaturday'))
			$skybill->service = '6';
		// Called from export admin
		if (Tools::getIsset('orders') && Chronopostfood::isSaturdayOptionApplicable())
			$skybill->service = '6';
		
		// Called from orders pane
		if (Tools::getIsset('orderid') && Chronopostfood::isSaturdayOptionApplicable())
			$skybill->service = '6';

		// Could be shipping for saturday but is not
		if (Chronopostfood::gettingReadyForSaturday() && $skybill->service != '6') $skybill->service = '957'; //
	}

	//echo Chronopostfood::gettingReadyForSaturday()." ".$skybill->service;

	$skybill->shipDate = date('Y-m-d\TH:i:s');
	$skybill->shipHour = date('H');

	// weight 0 when multishipping
	$skybill->weight = 0;
	// Only 1 skybill, put real weight. 
	if ($totalnb == 1) $skybill->weight = $o->getTotalWeight() * Configuration::get('CHRONOFOOD_GENERAL_WEIGHTCOEF');

	$skybill->weightUnit = 'KGM';


	$skybillParams = new skybillParamsValue();
	$skybillParams->mode = Configuration::get('CHRONOFOOD_GENERAL_PRINTMODE');
	$scheduledParams = new scheduledValue();
	$scheduledParams->expirationDate = $orderExpirationDate->format('Y-m-d\TH:i:s');

	//$params->esdValue = $esd;
	$params->headerValue = $header;
	$params->shipperValue = $shipper;
	$params->customerValue = $customer;
	$params->recipientValue = $recipient;
	$params->refValue = $ref;
	$params->skybillValue = $skybill;

	$params->skybillParamsValue = $skybillParams;

	$params->scheduledValue = $scheduledParams;

    $client = new SoapClient("https://www.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl", array('trace' => true));
    try {
        $response = $client->shippingV3($params);
        $r = $response->return;
    } catch (SoapFault $fault) {
        die($fault->faultstring);
    }

/*
	var_dump($params);
	var_dump($service->shipping($params)->return);
*/

	if ($r->errorCode != 0)
		return null;

	// MAIL::SEND is bugged in 1.5 !
	// http://forge.prestashop.com/browse/PNM-754 (Unresolved as of 2013-04-15)
	// Context fix (it's that easy)
	Context::getContext()->link = new Link(); 

	if ($isReturn)
	{
		$customer = new Customer($o->id_customer);        
		Mail::Send($o->id_lang, 'return', 'Lettre de transport Chronopost pour le retour de votre commande', 
			array('{id_order}' => $o->id, '{firstname}' => $customer->firstname,
			'{lastname}' => $customer->lastname), $customer->email, 
			$customer->firstname.' '.$customer->lastname, null, null, 
			array('content' => $r->skybill, 'mime' => 'application/pdf', 'name' => $r->skybillNumber.'.pdf'), 
			null, _CF_MYDIR_.'/mails/', true);
	}
	else
	{
	    if ($r->skybillNumber) {
            // Store LT for history
            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'chronofood_lt_history` VALUES (
                    '.(int)$o->id.', 
                    "'.pSQL($r->skybillNumber).'", 
                    "'.pSQL($skybill->productCode).'",
                    "'.pSQL($recipient->recipientZipCode).'",
                    "'.pSQL($recipient->recipientCountry).'",
                    "'.(isset($skybill->insuredValue) ? (int)$skybill->insuredValue : 0).'",
                    "'.pSQL($recipient->recipientCity).'"
                )');

            // Save expiration date
            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'chronofood_order_expiration_date` VALUES (
				'.(int)$o->id.', 
				"'.pSQL($scheduledParams->expirationDate).'" 
			)
			    ON DUPLICATE KEY UPDATE expiration_date = "'.pSQL($scheduledParams->expirationDate).'" 
			');

            Chronopostfood::trackingStatus($o->id, $r->skybillNumber);
        }
	}

/*
	header('Content-type: application/pdf');
header('Content-Disposition: attachment; filename="downloaded.pdf"');

	echo $r->skybill; 
	die();*/
	return $r->skybill;
}
